#!/bin/bash
#===========================================================
#           SOCflash Utility
#===========================================================

# Configuration
BMC_FILENAME=allv259.bin

# sub-function
start_openipmi_service () {
	FAILURE=false
	
	
	cat /etc/*release | grep Ubuntu
	if [ $? -eq 0 ]; then
		OStype_Ubuntu=true
		OStype_RedHat=false
	else
		OStype_Ubuntu=false
		OStype_RedHat=true		# Red Hat/ CentOS/ SLES
	fi
	
	#cat /etc/*release | grep Solaris
	#if [ $? -eq 0 ]; then
	#	OStype_Solaris=true
	#	echo "This utility is incompatible with Solaris.  Please run under Ubuntu or Red Hat."
	#	exit 3
	#else
	#	OStype_Solaris=false
	#fi
	
	#cat /etc/*release | grep "Red Hat"
	#if [ $? -eq 0 ]; then
	#	OStype_RedHat=true
	#else
	#	OStype_RedHat=false
	#fi
	#
	#cat /etc/*release | grep "CentOS"
	#if [ $? -eq 0 ]; then
	#	OStype_RedHat=true
	#else
	#	OStype_RedHat=false
	#fi
	
	ipmirunning=false
	
	if ( $OStype_Ubuntu ); then
		for ((i=0; i<3; i++))
		do
			modprobe ipmi_msghandler
			modprobe ipmi_devintf
			modprobe ipmi_si
			if [ $? -eq 0 ]; then
				ipmirunning=true
			break
			fi
			echo "openipmi service did not start; waiting 5 seconds to retry."
			sleep 5
		done
		if ( ! $ipmirunning ); then
			echo "IPMI service failed to start.  Please see that the service is started and re-run this script."
			FAILURE=true
		fi
	fi
	
	#if ( $OStype_Solaris ); then
	#  for ((i=0; i<3; i++))
	#  do
	#    # --- put command to start Solaris ipmi service here ---
	#    if [ $? -eq 0 ]; then
	#      ipmirunning=true
	#      break
	#    fi
	#    echo "ipmi service did not start; waiting 5 seconds to retry."
	#	sleep 5
	#  done
	#  if ( ! $ipmirunning ); then
	#    echo "IPMI service failed to start.  Please see that the service is started and re-run this script."
	#    FAILURE=true
	#  fi
	#fi
	
	
	if ( $OStype_RedHat ); then
		for ((i=0; i<3; i++))
		do
			#/etc/init.d/ipmi start
			service ipmi start
			if [ $? -eq 0 ]; then
				ipmirunning=true
				break
			fi
			echo "ipmi service did not start; waiting 5 seconds to retry."
			sleep 5
		done
		if ( ! $ipmirunning ); then
			echo "IPMI service failed to start.  Please see that the service is started and re-run this script."
			FAILURE=true
		fi
	fi
	
	if ( ! $ipmirunning ); then
			echo "IPMI service doesn't start.  Please see that the service is started and re-run this script."
			FAILURE=true
	fi
	
	# When IPMI service fails, stop this script
	if [ $FAILURE == true ] ; then
		exit 1
	fi
}

# $1: socflash_x64 or socflash
programming_bmc () {
	echo "============================================="
	echo "Updating Bootloader..."
	echo "============================================="
	./$1 if=$BMC_FILENAME skip=0 offset=0 count=0x30000
	echo "============================================="
	sleep 5
	echo "Updating Kernel..."
	echo "============================================="
	./$1 if=$BMC_FILENAME skip=0x100000 offset=0x100000 count=0x1f0000
	echo "============================================="
	sleep 5
	echo "Updating Root file system..."
	echo "============================================="
	./$1 if=$BMC_FILENAME skip=0x300000 offset=0x300000 count=0x1b00000
	echo "============================================="
	echo "Done."
}

update_bmc () {

	for ((i=1; i<=10; i++));
	do
		echo "============================================="
		echo " $i cycles"
		echo "============================================="
		programming_bmc $1
		echo "Waiting 180 seconds when BMC initiate"
		echo "Please do not press any key while counting down."
		sleep 180
		echo -n "Check Get Device ID..."
		ipmitool raw 6 1 1>/dev/null
		if [ $? -eq 0 ]; then
		echo "Pass."
		Check_code_base
		./settime.bin
		exit 0
		fi
		echo "Failed."
	done
}

MACflash () {
ipmitool lan set 01 macaddr $1
}

Check_code_base () {
if [ "$MajorVersion1" -gt "1" ]; then
	if [ "$MajorVersion1" -gt "4" ]; then
		if [ "$MinorVersion1" -gt "9" ]; then
			echo "No need to relieve bonding mode. "
			echo "Will update latest SDR and cold reset BMC. Please wait 180 seconds for BMC initiate"
			echo "Please do not press any key while counting down."
			ipmitool raw 0x30 0x5B 2> /dev/null
			ipmitool raw 0x06 0x02 2> /dev/null
			sleep 180
		else
			ipmitool raw 0x30 0x5B 2> /dev/null
			Relieve_bonding
		fi
	else
		if [ "$MinorVersion1" -gt "51" ]; then
			echo "No need to relieve bonding mode and update latest SDR. "
		else
			echo "No need to relieve bonding mode. "
			echo "Will update latest SDR and cold reset BMC. Please wait 180 seconds for BMC initiate"
			echo "Please do not press any key while counting down."
			ipmitool raw 0x30 0x5B 2> /dev/null
			ipmitool raw 0x06 0x02 2> /dev/null
			sleep 180
		fi
	fi
else
	ipmitool raw 0x30 0x5B > /dev/null
	Relieve_bonding
fi
}

Preserve_configuration () {
MajorVersion1=$(ipmitool raw 0x06 0x01 | cut -d ' ' -f 4)
MinorVersion1=$(ipmitool raw 0x06 0x01 | cut -d ' ' -f 5)
GetDeviceID1=$(ipmitool raw 0x06 0x01)
MAC=$(ipmitool lan print 01 | grep "MAC Address" | cut -d ':' -f 2-7)
ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
NICMode=$(ipmitool raw 0x30 0x02 0x$ReservationSh 0x02 0x01 0x00 0x00 0x00 0x01 | cut -d ' ' -f 6)
IPSource=$(ipmitool raw  0xc 0x2 0x1 0x4 0x0 0x0 | cut -d ' ' -f 3)
IP1=$(ipmitool raw  0xc 0x2 0x1 0x3 0x0 0x0 | cut -d ' ' -f 3)
IP2=$(ipmitool raw  0xc 0x2 0x1 0x3 0x0 0x0 | cut -d ' ' -f 4)
IP3=$(ipmitool raw  0xc 0x2 0x1 0x3 0x0 0x0 | cut -d ' ' -f 5)
IP4=$(ipmitool raw  0xc 0x2 0x1 0x3 0x0 0x0 | cut -d ' ' -f 6)
MASK1=$(ipmitool raw  0xc 0x2 0x1 0x6 0x0 0x0 | cut -d ' ' -f 3)
MASK2=$(ipmitool raw  0xc 0x2 0x1 0x6 0x0 0x0 | cut -d ' ' -f 4)
MASK3=$(ipmitool raw  0xc 0x2 0x1 0x6 0x0 0x0 | cut -d ' ' -f 5)
MASK4=$(ipmitool raw  0xc 0x2 0x1 0x6 0x0 0x0 | cut -d ' ' -f 6)
D4GWay1=$(ipmitool raw  0xc 0x2 0x1 0xc 0x0 0x0 | cut -d ' ' -f 3)
D4GWay2=$(ipmitool raw  0xc 0x2 0x1 0xc 0x0 0x0 | cut -d ' ' -f 4)
D4GWay3=$(ipmitool raw  0xc 0x2 0x1 0xc 0x0 0x0 | cut -d ' ' -f 5)
D4GWay4=$(ipmitool raw  0xc 0x2 0x1 0xc 0x0 0x0 | cut -d ' ' -f 6)
}

Relieve_bonding () {
echo "Restore LAN configuration to Default"
ipmitool raw 0x30 0x04 0xe2
sleep 30

echo "Cold reset BMC"
ipmitool raw 0x06 0x02
sleep 90

echo "Change Lan port from Dedicated NIC to Shared NIC"
ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
ipmitool raw 0x30 0x03 0x$ReservationSh 0x02 0x01 0x00 0x00 0x00 0x01 0x00
sleep 30

echo "Select Shared NIC 2"
ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
ipmitool raw 0x30 0x03 0x$ReservationSh 0x02 0x02 0x00 0x00 0x00 0x01 0x02
sleep 20

echo "Enter manufacturing test mode"
ipmitool raw 0x06 0x05 0x64 0x52 0x45 0x4d 0x24 0x73 0x74 0x38
sleep 5

echo "Flash BMC MAC with Shared NIC 2..."
MACflash $MAC
sleep 10
		
if [ "$NICMode" -eq "01" ]; then
	echo "Change Lan port to Dedicated NIC"
	ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
	ipmitool raw 0x30 0x03 0x$ReservationSh 0x02 0x01 0x00 0x00 0x00 0x01 0x01
	sleep 20
else
	echo "Restore Lan Port to Default: Shared NIC 1"
	ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
	ipmitool raw 0x30 0x03 0x$ReservationSh 0x02 0x01 0x00 0x00 0x00 0x01 0x00
	sleep 20
	ReservationSh=$(ipmitool raw 0x30 0x01 | cut -d ' ' -f 2)
	ipmitool raw 0x30 0x03 0x$ReservationSh 0x02 0x02 0x00 0x00 0x00 0x01 0x01
	sleep 30
fi
if [ "$IPSource" -eq "01" ]; then
	sleep 10
	echo "Set IP Source to static IP"
	ipmitool raw 0x0c 0x01 0x01 0x04 0x$IPSource
	sleep 10
	echo "Set static IP address"
	ipmitool raw 0x0c 0x01 0x01 0x03 0x$IP1 0x$IP2 0x$IP3 0x$IP4
	sleep 10
	echo "Set subnet mask"
	ipmitool raw 0x0c 0x01 0x01 0x06 0x$MASK1 0x$MASK2 0x$MASK3 0x$MASK4
	sleep 10
	echo "Set Default Gateway Address"
	ipmitool raw 0x0c 0x01 0x01 0x0c 0x$D4GWay1 0x$D4GWay2 0x$D4GWay3 0x$D4GWay4
	sleep 10
	ipmitool raw 0x0c 0x01 0x01 0x04 0x$IPSource
fi
echo "...done"

echo "Cold reset to exit manufacture mode"
ipmitool raw 0x06 0x02 > /dev/null
echo "Waiting 180 seconds when BMC initiate"
echo "Please do not press any key while counting down."
sleep 180
}

preserve_conf_socflash_update () {
	start_openipmi_service
	
	CMD=$(uname -m | cut -c 1-6)
	
	if [ "$CMD" = "x86_64" ]; then
	chmod 777 socflash_x64
	update_bmc socflash_x64
	
	else
	chmod 777 socflash
	update_bmc socflash
	fi
}

socflash_update () {
	start_openipmi_service
	
	CMD=$(uname -m | cut -c 1-6)
	for ((i=1; i<=10; i++));
	do
		echo "============================================="
		echo " $i cycles"
		echo "============================================="
		
		if [ "$CMD" = "x86_64" ]; then
			chmod 777 socflash_x64
			./socflash_x64 if=$BMC_FILENAME skip=0 offset=0 count=0x30000
			sleep 5
			./socflash_x64 if=$BMC_FILENAME skip=0x40000 offset=0x40000
		
		else
			chmod 777 socflash
			./socflash if=$BMC_FILENAME skip=0 offset=0 count=0x30000
			sleep 5
			./socflash if=$BMC_FILENAME skip=0x40000 offset=0x40000
		fi

		echo "Waiting 180 seconds"
		echo "Please do not press any key while counting down."
		sleep 180
		echo -n "Check Get Device ID..."
		ipmitool raw 6 1 1>/dev/null
		if [ $? -eq 0 ]; then
		echo "Pass."
		./settime.bin
		exit 0
		fi
		echo "Failed."
	done
}

# Main
service ipmi start
case "$1" in
"-s")
	echo "Do preserve config."
	Preserve_configuration
	preserve_conf_socflash_update
	;;
*)
	echo "Do not preserve config."
	socflash_update
	;;
esac

exit 0

